@@ -1,6 +1,9 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2015 Andreas Enge <andreas@enge.fr>
 ;;; Copyright © 2014 John Darrington <jmd@gnu.org>
+;;; Copyright © 2015 Andreas Enge <andreas@enge.fr>
+;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2017 Alex Griffin <a@ajgrf.com>
+;;; Copyright © 2017 Thomas Danckaert <post@thomasdanckaert.be>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -21,10 +24,11 @@
   #:use-module (guix build-system gnu)
   #:use-module (guix download)
   #:use-module ((guix licenses)
-                #:select (gpl2+ lgpl2.1+ mpl1.1 mpl2.0
+                #:select (gpl2+ lgpl2.1+ lgpl3+ mpl1.1 mpl2.0
                           non-copyleft x11-style))
   #:use-module (guix packages)
   #:use-module (guix utils)
+  #:use-module (gnu packages)
   #:use-module (gnu packages autotools)
   #:use-module (gnu packages base)
   #:use-module (gnu packages bison)
@@ -35,7 +39,7 @@
   #:use-module (gnu packages curl)
   #:use-module (gnu packages cyrus-sasl)
   #:use-module (gnu packages databases)
-  #:use-module (gnu packages doxygen)
+  #:use-module (gnu packages documentation)
   #:use-module (gnu packages flex)
   #:use-module (gnu packages fontutils)
   #:use-module (gnu packages ghostscript)
@@ -67,20 +71,20 @@
 (define-public ixion
   (package
     (name "ixion")
-    (version "0.9.1")
+    (version "0.12.2")
     (source
      (origin
       (method url-fetch)
       (uri (string-append "http://kohei.us/files/ixion/src/libixion-"
                           version ".tar.xz"))
       (sha256 (base32
-               "18g3nk29ljiqbyi0ml49j2x3f3xrqckdm9i66sw5fxnj7hb5rqvp"))))
+               "1bnsqbxpbijwbg42rrqq6mz06wvcxjpl0gjdzwyilkmv6s400i4b"))))
     (build-system gnu-build-system)
     (native-inputs
      `(("pkg-config" ,pkg-config)))
      (inputs
       `(("mdds" ,mdds)
-        ("python" ,python-2))) ; looks for python.pc, not python3.pc
+        ("python" ,python)))
     (home-page "https://gitlab.com/ixion/ixion")
     (synopsis "General purpose formula parser and interpreter")
     (description "Ixion is a library for calculating the results of formula
@@ -92,20 +96,21 @@ their dependencies automatically upon calculation.")
 (define-public orcus
   (package
     (name "orcus")
-    (version "0.7.1")
+    (version "0.12.1")
     (source
      (origin
       (method url-fetch)
       (uri (string-append "http://kohei.us/files/" name "/src/lib"
                           name "-" version ".tar.xz"))
       (sha256 (base32
-               "0hva4qalg3dk6n1118ncr5fy8cqnj2f7fwldw7aa04124rj6p104"))))
+               "171bmqa9hkk4xygz20qda5900rs4kq9fgl424ldkxlj4d733dffi"))))
     (build-system gnu-build-system)
     (native-inputs
      `(("pkg-config" ,pkg-config)))
      (inputs
       `(("ixion" ,ixion)
         ("mdds" ,mdds)
+        ("python" ,python)
         ("zlib" ,zlib)))
     (home-page "https://gitlab.com/orcus/orcus")
     (synopsis "File import filter library for spreadsheet documents")
@@ -125,21 +130,24 @@ CSV, CSS and XML.")
      (origin
       (method url-fetch)
       (uri (string-append "mirror://sourceforge/libwpd/" name "/" name "-"
-                          version ".tar.xz"))
+                          version "/" name "-" version ".tar.xz"))
       (sha256 (base32
                "03ygxyb0vfjv8raif5q62sl33b54wkr5rzgadb8slijm6k281wpn"))))
     (build-system gnu-build-system)
     (native-inputs
-     `(("boost" ,boost)
-       ("cppunit" ,cppunit)
+     `(("cppunit" ,cppunit)
        ("doxygen" ,doxygen)
        ("pkg-config" ,pkg-config)))
     (inputs
      `(("zlib" ,zlib)))
+    (propagated-inputs                  ; Referenced by .la files
+     `(("boost" ,boost)))
     (arguments
      ;; avoid triggering configure errors by simple inclusion of boost headers
-     `(#:configure-flags '("--disable-werror")))
-    (home-page "http://sourceforge.net/p/libwpd/wiki/librevenge/")
+     `(#:configure-flags '("--disable-werror"
+                           ;; Avoid undefined library references
+                           "LDFLAGS=-lboost_system")))
+    (home-page "https://sourceforge.net/p/libwpd/wiki/librevenge/")
     (synopsis "Document importer for office suites")
     (description "Librevenge is a base library for writing document import
 filters.  It has interfaces for text documents, vector graphics,
@@ -153,8 +161,8 @@ spreadsheets and presentations.")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/libwpd/" name "/" name "-"
-                          version ".tar.xz"))
+      (uri (string-append "mirror://sourceforge/" name "/" name "/"
+                          name "-" version "/" name "-" version ".tar.xz"))
       (sha256 (base32
                "0b6krzr6kxzm89g6bapn805kdayq70hn16n5b5wfs2lwrf0ag2wx"))))
     (build-system gnu-build-system)
@@ -215,8 +223,8 @@ ZVR (simple compressed text format).")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/libwpg/" name "/" name "-"
-                          version ".tar.xz"))
+      (uri (string-append "mirror://sourceforge/" name "/" name "/"
+                          name "-" version "/" name "-" version ".tar.xz"))
       (sha256 (base32
                "097jx8a638fwwfrzf6v29r1yhc34rq9526py7wf0ck2z4fcr2w3g"))))
     (build-system gnu-build-system)
@@ -237,14 +245,15 @@ working with graphics in the WPG (WordPerfect Graphics) format.")
 (define-public libcmis
   (package
     (name "libcmis")
-    (version "0.5.0")
+    (version "0.5.1")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/" name "/" name "-"
-                          version ".tar.gz"))
-      (sha256 (base32
-               "1dprvk4fibylv24l7gr49gfqbkfgmxynvgssvdcycgpf7n8h4zm8"))))
+      (uri (string-append "https://github.com/tdf/libcmis/releases/download/v"
+                          version "/libcmis-" version ".tar.gz"))
+      (sha256
+       (base32
+        "03kvl8ywsv5qrxblf0m6955mmvl5q2zpb6vj51vs7ayvxhidzjva"))))
     (build-system gnu-build-system)
     (native-inputs
      `(("cppunit" ,cppunit)
@@ -269,7 +278,7 @@ working with graphics in the WPG (WordPerfect Graphics) format.")
           ;; fails without the following flag.
           (string-append "--with-boost="
                          (assoc-ref %build-inputs "boost")))))
-    (home-page "http://sourceforge.net/projects/libcmis/")
+    (home-page "https://github.com/tdf/libcmis")
     (synopsis "CMIS client library")
     (description "LibCMIS is a C++ client library for the CMIS interface.  It
 allows C++ applications to connect to any ECM behaving as a CMIS server such
@@ -341,22 +350,31 @@ CorelDRAW documents of all versions.")
 (define-public libetonyek
   (package
     (name "libetonyek")
-    (version "0.1.3")
+    (version "0.1.6")
     (source
      (origin
       (method url-fetch)
       (uri (string-append "http://dev-www.libreoffice.org/src/" name "/"
                           name "-" version ".tar.xz"))
       (sha256 (base32
-               "0mghaqzj0qqza8z1gzprw62702adlww4kgdzynj5qpxxc9m2f4py"))))
+               "0y60vi1plyq69fqbcjnc0v8mvcjqjsl1ry6rmb3bq3q7j8a2fm6z"))
+      (patches (search-patches "libetonyek-build-with-mdds-1.2.patch"))))
     (build-system gnu-build-system)
+    (arguments
+     `(#:configure-flags '("--with-mdds=1.2")
+       #:phases (modify-phases %standard-phases
+                  (add-before 'configure 'autoreconf
+                              (lambda _ (system* "autoreconf"))))))
     (native-inputs
      `(("cppunit" ,cppunit)
        ("doxygen" ,doxygen)
        ("glm" ,glm)
        ("gperf" ,gperf)
+       ("liblangtag" ,liblangtag)
        ("mdds" ,mdds)
-       ("pkg-config" ,pkg-config)))
+       ("pkg-config" ,pkg-config)
+       ("autoconf" ,autoconf) ; due to patch
+       ("automake" ,automake)))
     (propagated-inputs ; in Requires or Requires.private field of .pkg
      `(("librevenge" ,librevenge)
        ("libxml2" ,libxml2)))
@@ -368,6 +386,33 @@ CorelDRAW documents of all versions.")
 Apple Keynote documents.  It currently supports Keynote versions 2 to 5.")
     (license mpl2.0)))
 
+(define-public liblangtag
+  (package
+    (name "liblangtag")
+    (version "0.5.8")
+    (source
+      (origin
+        (method url-fetch)
+        (uri (string-append "https://bitbucket.org/tagoh/liblangtag/downloads/"
+                            name "-" version ".tar.bz2"))
+        (sha256
+         (base32
+          "1akf0d7yp29pv3j2pw2riii4n5kyjr9szc0y77khnx9zzr5zdqh8"))))
+    (build-system gnu-build-system)
+    (native-inputs
+     `(("libtool" ,libtool)
+       ("pkg-config" ,pkg-config)))
+    (inputs
+     `(("libxml2" ,libxml2)))
+    (home-page "http://tagoh.bitbucket.org/liblangtag/")
+    (synopsis "Library to access tags for identifying languages")
+    (description "Liblangtag implements an interface to work with tags
+for identifying languages as described in RFC 5646.  It supports the
+extensions described in RFC6067 and RFC6497, and Extension T for
+language/locale identifiers as described in the Unicode CLDR
+standard 21.0.2.")
+    (license (list lgpl3+ mpl2.0)))) ; dual license
+
 (define-public libexttextcat
   (package
     (name "libexttextcat")
@@ -528,7 +573,7 @@ Microsoft Visio documents of all versions.")
     (arguments
      ;; avoid triggering configure errors by simple inclusion of boost headers
      `(#:configure-flags '("--disable-werror")))
-    (home-page "http://sourceforge.net/p/libwpd/wiki/libodfgen/")
+    (home-page "https://sourceforge.net/p/libwpd/wiki/libodfgen/")
     (synopsis "ODF (Open Document Format) library")
     (description "Libodfgen is a library for generating documents in the
 Open Document Format (ODF).  It provides generator implementations for all
@@ -543,8 +588,8 @@ text documents, vector drawings, presentations and spreadsheets.")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/" name "/" name "/"
-                          name "-" version ".tar.xz"))
+      (uri (string-append "mirror://sourceforge/" name "/" name "/" name "-"
+                          version "/" name "-" version ".tar.xz"))
       (sha256 (base32
                "1vx9h419fcfcs0yj071hsg9d2qvkacgca6052m8hv3h743cdmzil"))))
     (build-system gnu-build-system)
@@ -559,13 +604,36 @@ text documents, vector drawings, presentations and spreadsheets.")
     (arguments
      ;; avoid triggering configure errors by simple inclusion of boost headers
      `(#:configure-flags '("--disable-werror")))
-    (home-page "http://sourceforge.net/p/libmwaw/wiki/Home/")
+    (home-page "https://sourceforge.net/p/libmwaw/wiki/Home/")
     (synopsis "Import library for some old Macintosh text documents")
     (description "Libmwaw contains some import filters for old Macintosh
 text documents (MacWrite, ClarisWorks, ... ) and for some graphics and
 spreadsheet documents.")
     (license (list mpl2.0 lgpl2.1+)))) ; dual license
 
+(define-public libstaroffice
+  (package
+    (name "libstaroffice")
+    (version "0.0.3")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "https://github.com/fosnola/libstaroffice/releases/download/"
+                           version "/libstaroffice-" version ".tar.xz"))
+       (sha256 (base32
+                "1ii2wi3wr5npyz9gby1bjk8r4wyflpfpc6gx7mmqkhsc9c8frpmy"))))
+    (build-system gnu-build-system)
+    (inputs
+     `(("librevenge" ,librevenge)
+       ("zlib" ,zlib)))
+    (native-inputs
+     `(("pkg-config" ,pkg-config)))
+    (home-page "https://github.com/fosnola/libstaroffice")
+    (synopsis "Provides LibreOffice support for old StarOffice documents")
+    (description "@code{libstaroffice} is an import filter for the document formats
+from the old StarOffice (.sdc, .sdw, ...).")
+    (license (list mpl2.0 lgpl2.1+)))) ; dual license
+
 (define-public libwps
   (package
     (name "libwps")
@@ -574,7 +642,7 @@ spreadsheet documents.")
      (origin
       (method url-fetch)
       (uri (string-append "mirror://sourceforge/" name "/" name "/"
-                          name "-" version ".tar.xz"))
+                          name "-" version "/" name "-" version ".tar.xz"))
       (sha256 (base32
                "0nlrdk7di015l0sk0ivjdqs86zdcvf73p9z9s9ry5glyhrknzxjk"))))
     (build-system gnu-build-system)
@@ -595,24 +663,56 @@ spreadsheet documents.")
 Works word processor file format.")
     (license (list mpl2.0 lgpl2.1+)))) ; dual license
 
+(define-public libzmf
+  (package
+   (name "libzmf")
+   (version "0.0.1")
+   (source
+    (origin
+      (method url-fetch)
+      (uri (string-append "http://dev-www.libreoffice.org/src/libzmf/libzmf-"
+                          version ".tar.xz"))
+      (sha256 (base32
+               "0yp5l1b90xim506zmr3ljkn3qkvbc7qk3dnwq1snxdpr57m37xga"))))
+   (build-system gnu-build-system)
+   (inputs
+    `(("boost" ,boost)
+      ("icu4c" ,icu4c)
+      ("libpng" ,libpng)
+      ("librevenge" ,librevenge)
+      ("zlib" ,zlib)))
+    (native-inputs
+     `(("cppunit" ,cppunit)
+       ("doxygen" ,doxygen)
+       ("pkg-config" ,pkg-config)))
+    (home-page "https://wiki.documentfoundation.org/DLP/Libraries/libzmf")
+    (synopsis "Parses file format of Zoner Callisto/Draw documents")
+    (description "Libzmf is a library that parses the file format of Zoner
+Callisto/Draw documents.  Currently it only understands documents created by
+Zoner Draw version 4 and 5.")
+    (license mpl2.0)))
+
 (define-public hunspell
   (package
     (name "hunspell")
-    (version "1.3.3")
+    (version "1.5.4")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/" name "/"
-                          name "-" version ".tar.gz"))
+      (uri (string-append "https://github.com/hunspell/hunspell/archive/v"
+                          version ".tar.gz"))
       (sha256 (base32
-               "0v14ff9s37vkh45diaddndcrj0hmn67arh8xh8k79q9c1vgc1cm7"))))
+               "0ngwk18dwd8p5a5f20h2jlgrz9wbc1k189mmmprb2zmqwfi02b45"))
+      (file-name (string-append name "-" version ".tar.gz"))))
     (build-system gnu-build-system)
+    (inputs
+     `(("perl" ,perl)))
     (home-page "http://hunspell.sourceforge.net/")
     (synopsis "Spell checker")
     (description "Hunspell is a spell checker and morphological analyzer
 library and program designed for languages with rich morphology and complex
 word compounding or character encoding.")
-    ;; triple license, including "mpl1.1 or later"
+    ;; Triple license, including "mpl1.1 or later".
     (license (list mpl1.1 gpl2+ lgpl2.1+))))
 
 (define-public hyphen
@@ -622,7 +722,8 @@ word compounding or character encoding.")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/hunspell/"
+      (uri (string-append "mirror://sourceforge/hunspell/Hyphen/"
+                          (version-major+minor version) "/"
                           name "-" version ".tar.gz"))
       (sha256 (base32
                "01ap9pr6zzzbp4ky0vy7i1983fwyqy27pl0ld55s30fdxka3ciih"))))
@@ -643,7 +744,7 @@ patterns, which are pre-processed by a perl script.")
     (source
      (origin
       (method url-fetch)
-      (uri (string-append "mirror://sourceforge/hunspell/"
+      (uri (string-append "mirror://sourceforge/hunspell/MyThes/" version "/"
                           name "-" version ".tar.gz"))
       (sha256 (base32
                "0prh19wy1c74kmzkkavm9qslk99gz8h8wmjvwzjc6lf8v2az708y"))))
@@ -668,15 +769,15 @@ and to return information on pronunciations, meanings and synonyms.")
     (method url-fetch)
     (uri
       (string-append
-        "http://dev-www.libreoffice.org/src/"
-        "1f24ab1d39f4a51faf22244c94a6203f-xmlsec1-1.2.14.tar.gz"))
+       "http://dev-www.libreoffice.org/src/"
+       "86b1daaa438f5a7bea9a52d7b9799ac0-xmlsec1-1.2.23.tar.gz"))
     (sha256 (base32
-              "0jnxxygg6z5zi6za94dvxmg1bfar1wh8p5xa2bzbha0qcn2m02ir"))))
+             "17qfw5crkqn4v6xbkjxrjvcccfc00dy053892wrwv54qdk8n7m21"))))
 
 (define-public libreoffice
   (package
     (name "libreoffice")
-    (version "5.0.0.2")
+    (version "5.3.1.2")
     (source
      (origin
       (method url-fetch)
@@ -685,7 +786,7 @@ and to return information on pronunciations, meanings and synonyms.")
           "http://download.documentfoundation.org/libreoffice/src/"
           (version-prefix version 3) "/libreoffice-" version ".tar.xz"))
       (sha256 (base32
-               "0mzvxc4i5999xzmhgwrfvpvyx5772jylx5mary86drrld8klq7py"))))
+               "1zsl0z0i8pw532x2lmwd64ms6igibkkjhwf01zmm2kpnr9ycsijp"))))
     (build-system gnu-build-system)
     (native-inputs
      `(;; autoreconf is run by the LibreOffice build system, since after
@@ -697,7 +798,7 @@ and to return information on pronunciations, meanings and synonyms.")
        ("cppunit" ,cppunit)
        ("flex" ,flex)
        ("pkg-config" ,pkg-config)
-       ("python" ,python)
+       ("python" ,python-wrapper)
        ("which" ,which)))
     (inputs
      `(("bluez" ,bluez)
@@ -728,6 +829,7 @@ and to return information on pronunciations, meanings and synonyms.")
        ("libmwaw" ,libmwaw)
        ("libodfgen" ,libodfgen)
        ("libpagemaker" ,libpagemaker)
+       ("libstaroffice" ,libstaroffice)
        ("libvisio" ,libvisio)
        ("libwpg" ,libwpg)
        ("libwps" ,libwps)
@@ -735,6 +837,7 @@ and to return information on pronunciations, meanings and synonyms.")
        ("libxrender" ,libxrender)
        ("libxslt" ,libxslt)
        ("libxt" ,libxt)
+       ("libzmf" ,libzmf)
        ("lpsolve" ,lpsolve)
        ("mdds" ,mdds)
        ("mythes" ,mythes)
@@ -757,8 +860,7 @@ and to return information on pronunciations, meanings and synonyms.")
        ("xmlsec-src" ,xmlsec-src-libreoffice)
        ("zip" ,zip)))
     (arguments
-     `(#:parallel-build? #f ; Otherwise the build fails.
-       #:tests? #f ; Building the tests already fails.
+     `(#:tests? #f ; Building the tests already fails.
        #:make-flags '("build-nocheck") ; Do not build unit tests, which fails.
        #:phases
          (modify-phases %standard-phases
@@ -768,15 +870,14 @@ and to return information on pronunciations, meanings and synonyms.")
                  (substitute*
                    (list "sysui/CustomTarget_share.mk"
                          "solenv/gbuild/gbuild.mk"
-                         "solenv/gbuild/platform/unxgcc.mk"
-                         "external/libxmlsec/xmlsec1-oldlibtool.patch")
+                         "solenv/gbuild/platform/unxgcc.mk")
                    (("/bin/sh") (which "bash")))
                  (mkdir "external/tarballs")
                  (symlink
                    xmlsec
                    (string-append "external/tarballs/"
-                     "1f24ab1d39f4a51faf22244c94a6203f-"
-                     "xmlsec1-1.2.14.tar.gz"))
+                                  "86b1daaa438f5a7bea9a52d7b9799ac0-"
+                                  "xmlsec1-1.2.23.tar.gz"))
                  ;; The following is required for building xmlsec from the
                  ;; unpatched external tarball; since "configure" starts with
                  ;; "/bin/sh", it needs to be executed by a command invoking
@@ -787,14 +888,16 @@ and to return information on pronunciations, meanings and synonyms.")
                    (("./configure") "$(CONFIG_SHELL) ./configure" ))
                  #t)))
            (add-after 'install 'bin-install
-             ;; Create a symlink bin/soffice to the executable script.
+             ;; Create 'soffice' and 'libreoffice' symlinks to the executable
+             ;; script.
              (lambda* (#:key outputs #:allow-other-keys)
                (let* ((out (assoc-ref outputs "out"))
-                      (bin (string-append out "/bin")))
+                      (bin (string-append out "/bin"))
+                      (soffice (string-append
+                                out "/lib/libreoffice/program/soffice")))
                  (mkdir bin)
-                 (symlink
-                   (string-append out "/lib/libreoffice/program/soffice")
-                   (string-append bin "/soffice")))
+                 (symlink soffice (string-append bin "/soffice"))
+                 (symlink soffice (string-append bin "/libreoffice")))
                #t)))
        #:configure-flags
         (list
@@ -805,6 +908,8 @@ and to return information on pronunciations, meanings and synonyms.")
           "--with-system-libs" ; enable all --with-system-* flags
           (string-append "--with-boost-libdir="
                          (assoc-ref %build-inputs "boost") "/lib")
+          ;; Avoid undefined symbols required by boost::spirit
+          "LDFLAGS=-lboost_system"
           ;; Avoid a dependency on ucpp.
           "--with-idlc-cpp=cpp"
           ;; The fonts require an external tarball (crosextrafonts).
@@ -817,6 +922,8 @@ and to return information on pronunciations, meanings and synonyms.")
           "--disable-coinmp"
           "--disable-firebird-sdbc" ; embedded firebird
           "--disable-gltf"
+          "--without-doxygen"
+          "--disable-gtk3"
           "--disable-liblangtag")))
     (home-page "https://www.libreoffice.org/")
     (synopsis "Office suite")
