@@ -1,6 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012, 2013, 2015 Ludovic Courtès <ludo@gnu.org>
-;;; Copyright © 2014 Eric Bavier <bavier@member.fsf.org>
+;;; Copyright © 2012, 2013, 2015, 2016 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2014, 2016 Eric Bavier <bavier@member.fsf.org>
 ;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -32,18 +32,17 @@
 (define-public texinfo
   (package
     (name "texinfo")
-    (version "6.0")
+    (version "6.3")
     (source (origin
               (method url-fetch)
               (uri (string-append "mirror://gnu/texinfo/texinfo-"
                                   version ".tar.xz"))
               (sha256
                (base32
-                "1r3i6jyynn6ab45fxw5bms8mflk9ry4qpj6gqyry72vfd5c47fhi"))))
+                "0fpr9kdjjl6nj2pc50k2zr7134hvqz8bi8pfqa7131a9lpzz6v14"))))
     (build-system gnu-build-system)
     (native-inputs `(("procps" ,procps)))  ;one of the tests needs pgrep
     (inputs `(("ncurses" ,ncurses)
-              ("xz" ,xz)
               ("perl" ,perl)))
 
     (native-search-paths
@@ -52,7 +51,7 @@
             (variable "INFOPATH")
             (files '("share/info")))))
 
-    (home-page "http://www.gnu.org/software/texinfo/")
+    (home-page "https://www.gnu.org/software/texinfo/")
     (synopsis "The GNU documentation format")
     (description
      "Texinfo is the official documentation format of the GNU project.  It
@@ -90,6 +89,41 @@ is on expressing the content semantically, avoiding physical markup commands.")
     (native-inputs '())
     (inputs `(("ncurses" ,ncurses) ("xz" ,xz)))))
 
+(define-public info-reader
+  ;; The idea of this package is to have the standalone Info reader without
+  ;; the dependency on Perl that 'makeinfo' drags.
+  (package
+    (inherit texinfo)
+    (name "info-reader")
+    (arguments
+     `(#:disallowed-references ,(assoc-ref (package-inputs texinfo)
+                                           "perl")
+
+       #:modules ((ice-9 ftw) (srfi srfi-1)
+                  ,@%gnu-build-system-modules)
+
+       #:phases (modify-phases %standard-phases
+                  (add-after 'install 'keep-only-info-reader
+                    (lambda* (#:key outputs #:allow-other-keys)
+                      ;; Remove everything but 'bin/info' and associated
+                      ;; files.
+                      (define (files)
+                        (scandir "." (lambda (file)
+                                       (not (member file '("." ".."))))))
+
+                      (let ((out (assoc-ref outputs "out")))
+                        (with-directory-excursion out
+                          (for-each delete-file-recursively
+                                    (fold delete (files) '("bin" "share"))))
+                        (with-directory-excursion (string-append out "/bin")
+                          (for-each delete-file (delete "info" (files))))
+                        (with-directory-excursion (string-append out "/share")
+                          (for-each delete-file-recursively
+                                    (fold delete (files)
+                                          '("info" "locale"))))
+                        #t))))))
+    (synopsis "Standalone Info documentation reader")))
+
 (define-public texi2html
   (package
     (name "texi2html")
@@ -102,8 +136,8 @@ is on expressing the content semantically, avoiding physical markup commands.")
                (base32
                 "1yprv64vrlcbksqv25asplnjg07mbq38lfclp1m5lj8cw878pag8"))
               (patches
-               (list (search-patch "texi2html-document-encoding.patch")
-                     (search-patch "texi2html-i18n.patch")))
+               (search-patches "texi2html-document-encoding.patch"
+                               "texi2html-i18n.patch"))
               (snippet
                ;; This file is modified by the patch above, but reset its
                ;; timestamp so we don't trigger the rule to update PO files,
@@ -112,6 +146,11 @@ is on expressing the content semantically, avoiding physical markup commands.")
                '(utime "texi2html.pl" 0 0 0 0))))
     (build-system gnu-build-system)
     (inputs `(("perl" ,perl)))
+    (arguments
+     ;; Tests fail because of warnings on stderr from Perl 5.22.  Adjusting
+     ;; texi2html.pl to avoid the warnings seems non-trivial, so we simply
+     ;; disable the tests.
+     '(#:tests? #f))
     (home-page "http://www.nongnu.org/texi2html/")
     (synopsis "Convert Texinfo to HTML")
     (description
